
var Inspectomatic = 
{
  mSelecting: false,
  mSelectDocs: null,
  mFlasher: null,

  selectByClick: function(aDocument)
  {
    if (this.mSelecting) {
      this.mSelecting = false;
      this.removeClickListeners();
    } else {
      this.mDocument = aDocument;
      // wait until after user releases the mouse after selecting this command from a UI element
      window.setTimeout("Inspectomatic.startSelectByClick()", 0);
    }
  },
  
  startSelectByClick: function()
  {    
    if (!this.mFlasher)
      this.mFlasher = Components.classes["@mozilla.org/inspector/flasher;1"]
                                .getService(Components.interfaces.inIFlasher);

    this.mSelecting = true;
    this.mSelectDocs = this.getAllDocuments(this.mDocument);

    for (var i = 0; i < this.mSelectDocs.length; ++i) {
      this.mSelectDocs[i].addEventListener("mousedown", this.onMouseDown, true);
      this.mSelectDocs[i].addEventListener("mouseup", this.onMouseUp, true);
      this.mSelectDocs[i].addEventListener("click", this.onClick, true);
    }
  },

  doSelectByClick: function(aTarget)
  {
    if (aTarget.nodeType == Node.TEXT_NODE)
      aTarget = aTarget.parentNode;
      
    this.mSelecting = false;
    
    InspectorWidget.openDOMInspector(aTarget);
  },

  removeClickListeners: function()
  {
    for (var i = 0; i < this.mSelectDocs.length; ++i) {
      this.mSelectDocs[i].removeEventListener("mousedown", this.onMouseDown, true);
      this.mSelectDocs[i].removeEventListener("mouseup", this.onMouseUp, true);
      this.mSelectDocs[i].removeEventListener("click", this.onClick, true);
    }
  },

  ////////////////////////////////////////////////////////////////////////////
  //// Listeners

  onMouseDown: function(aEvent)
  {
    aEvent.stopPropagation();
    aEvent.preventDefault();

    var node = aEvent.target;
    while (node && node.nodeType != Node.ELEMENT_NODE)
      node = node.parentNode;

    this.mCurrentNode = node;
    Inspectomatic.mFlasher.drawElementOutline(node, "aa0000", 4, true);
  },
  
  onMouseUp: function(aEvent)
  {
    aEvent.stopPropagation();
    aEvent.preventDefault();

    Inspectomatic.mFlasher.repaintElement(this.mCurrentNode);
  },

  onClick: function(aEvent)
  {
    aEvent.stopPropagation();
    aEvent.preventDefault();

    Inspectomatic.doSelectByClick(aEvent.target);
    Inspectomatic.removeClickListeners();
  },

  ////////////////////////////////////////////////////////////////////////////
  //// Uncategorized
  
  getAllDocuments: function(aDocument)
  {
    var results = [aDocument];
    this.findDocuments(aDocument, results);
    return results;
  },
  
  findDocuments: function(aDoc, aArray)
  {
    const kXULNSURI = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
    this.addKidsToArray(aDoc.getElementsByTagName("frame"), aArray);
    this.addKidsToArray(aDoc.getElementsByTagName("iframe"), aArray);
    this.addKidsToArray(aDoc.getElementsByTagNameNS(kXULNSURI, "browser"), aArray);
    this.addKidsToArray(aDoc.getElementsByTagNameNS(kXULNSURI, "tabbrowser"), aArray);
    this.addKidsToArray(aDoc.getElementsByTagNameNS(kXULNSURI, "editor"), aArray);
  },
  
  addKidsToArray: function(aKids, aArray)
  {
    for (var i = 0; i < aKids.length; ++i) {
      try {
        if (aKids[i].localName == "editor")
          aArray.push(aKids[i].editorShell.editorDocument);
        else
          aArray.push(aKids[i].contentDocument);
      } catch (ex) {
        // if we can't access the content document, skip it
      }
    }
  }
};
